unit RandomDlgs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, ExtCtrls, CPanel, Tools, TextCtrl;

type
  TRandomSetForm = class(TForm)
    Tabs: TPageControl;
    TabSheetSCRS: TTabSheet;
    ListViewSCR: TListBox;
    BitBtnCancel: TBitBtn;
    BitBtnOk: TBitBtn;
    CenterPanel1: TCenterPanel;
    BitBtnTest: TBitBtn;
    BitBtnSet: TBitBtn;
    BitBtnDel: TBitBtn;
    BitBtnAdd: TBitBtn;
    OpenDialogSCR: TOpenDialog;
    AboutSheet: TTabSheet;
    AboutText: TText;
    Text1: TText;
    ImageEG: TImage;
    CenterPanel2: TCenterPanel;
    MeinNameText: TLabel;
    DevelImage: TImage;
    Text2: TText;
    DavidEddingsNameText: TLabel;
    Image1: TImage;
    OrderSheet: TTabSheet;
    OrderListBox: TListBox;
    CenterPanel3: TCenterPanel;
    BitBtnRandom: TBitBtn;
    BitBtnSort: TBitBtn;
    BitBtnDown: TBitBtn;
    BitBtnUp: TBitBtn;
    procedure BitBtnAddClick(Sender: TObject);
    procedure BitBtnDelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtnSetClick(Sender: TObject);
    procedure BitBtnCancelClick(Sender: TObject);
    procedure BitBtnOkClick(Sender: TObject);
    procedure BitBtnTestClick(Sender: TObject);
    procedure ListViewClicked(Sender: TObject);
    procedure OrderListBoxClick(Sender: TObject);
    procedure BitBtnRandomClick(Sender: TObject);
    procedure BitBtnSortClick(Sender: TObject);
    procedure BitBtnUpClick(Sender: TObject);
    procedure BitBtnDownClick(Sender: TObject);
    procedure TabsChange(Sender: TObject);

  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  RandomSetForm: TRandomSetForm;

const
  sRandomGroup = 'PlayList';
  sGroupName = 'ListViewSCR';
  sCompanyName = 'Software Schmiede GbR';
  sApplicationName = 'Randomizing Screensavers';
  sIdScrnSaverNameCaption = 'Eindeutigen Namen festlegen';
  sIdScrnSaverNamePrompt = 'Name';

  msgNoScreenSavers = 'Sie mssen mindestens einen zu startenden Bildschirmschoner angeben';
  capNoScreenSavers = 'Kein Bildschirmschoner angegeben';
  msgRunError = 'Fehler beim Starten des Bildschirmschoners: ';
  capRunError = 'Fehler beim Starten des Bildschirmschoners';
implementation

{$R *.DFM}

uses
  Registry, Exes;

function Get16BitSCRName( const FileName : string ) : string;
var
         f : tFileStream;
         Header: record
           case integer of
             1: ( _Old: pOldExeHeader );
             2: ( _New: pNewExeHeader );
         end;
         s: ShortString;
         b: integer;

begin
  Result := '';
  if not Is16BitWindowsModule( FileName ) then
    raise EInvalidModule.Create( 'not a 16-bit windows library' );

  f := tFileStream.Create( FileName, fmOpenRead or fmShareDenyWrite );
  Header._Old := new( pOldExeHeader );
  f.Read( Header._Old^, sizeof( Header._Old^ ) );

  f.Seek( Header._Old^.NewExeAddr, soFromBeginning );
  dispose( Header._Old );
  Header._New := new( pNewExeHeader );
  f.Read( Header._New^, sizeof( tNewExeHeader ) );

  with Header._New^ do
  begin
    f.Seek( NonResidentTable, soFromBeginning );
    f.Read( b, 1 );
    SetLength( s, b );
    f.Read( s[1], b );
    Delete( s, 1, 10 );
    while ( ' ' = s[1] ) and ( 0 < Length( s ) ) do
      Delete( s, 1, 1 );
    if 0 = Length( s ) then
      Result := GetFileName( FileName )
    else
      Result := s;
  end;
  f.Free;
  dispose( Header._New );
end;

function GetScreenSaverName( const FileName : string; idName : string ) : string;
var
  tmp, tmp1, tmp1b: String;
  tmp2: PChar;
  module: LongInt;

begin
  Result := '';
  if ParamStr( 0 ) = FileName then
    exit;

  GetMem( tmp2, 256 );
  StrPCopy( tmp2, FileName );
  module := LoadLibrary( tmp2 );
  if 0 = module then
  begin
    if Is16BitWindowsModule( FileName ) then
    begin
      tmp1b := FileName;
      tmp1 := Get16BitSCRName( FileName );
      if 0 < Length( tmp1 ) then
        tmp := tmp1 + ' - "' + FileName + '"'
      else
        tmp := GetFileName( tmp ) + ' - "' + FileName + '"';
      if '' = idName then
      begin
        repeat
          tmp1 := InputBox( sIdScrnSaverNameCaption, sIdScrnSaverNamePrompt, GetFileName( FileName ) );
        until '' <> tmp1;
        tmp := tmp1 + ': ' + tmp;
      end else
        tmp := idName + ': ' + tmp;
    end else
    begin
      tmp := 'Error: Module "' + FileName + '" cannot be opened. File will be ignored.';
      StrPCopy( tmp2, tmp );
      Application.MessageBox( tmp2, 'Load Library has failed', mb_Ok OR mb_IconError );
      Result := '';
      exit;
    end;
  end else
  begin
    if 0 <> LoadString( module, 1, tmp2, 256 ) then
      tmp := StrPas( tmp2 ) + ' - "' + FileName + '"'
    else
      tmp := GetFileName( FileName ) + ' - "' + FileName + '"';
    FreeLibrary( module );
    if '' = idName then
      begin
        repeat
          tmp1 := InputBox( sIdScrnSaverNameCaption, sIdScrnSaverNamePrompt, GetFileName( FileName ) );
        until '' <> tmp1;
        tmp := tmp1 + ': ' + tmp;
      end else
        tmp := idName + ': ' + tmp;
  end;
  Result := tmp;
end;

procedure TRandomSetForm.BitBtnAddClick(Sender: TObject);
var
  Needed: Longint;
  Dir: String;
  i: integer;

const
  DirLen = 30;

begin
  with OpenDialogSCR do
  begin
    SetString(Dir,nil,DirLen);
    Needed := ExpandEnvironmentStrings( '%SystemRoot%', PChar(Dir), DirLen);
    if Needed > DirLen then
    begin
      SetString(Dir,nil,Needed);
      ExpandEnvironmentStrings( '%SystemRoot%', PChar(Dir), Needed);
    end;
    InitialDir := Dir;
    FileName := '*.scr';
    if Execute then
    begin
      { get true name of screen saver }
       for i := 0 to OpenDialogSCR.Files.Count - 1 do
       begin
         Dir := GetScreenSaverName( OpenDialogSCR.Files[i], '' );
         if '' <> Dir then
         begin
           ListViewSCR.Items.Add( Dir );
           OrderListBox.Items.Insert( Trunc( ( OrderListBox.Items.Count + 1 ) * SYSTEM.Random ), Copy( Dir, 1, Pos( ':', Dir ) - 1 ) );
         end;
       end;
    end;
  end;

  if OrderListBox.Items.Count = 0 then
    BitBtnOk.Enabled := False
  else
    BitBtnOk.Enabled := True;
end;

procedure TRandomSetForm.BitBtnDelClick(Sender: TObject);
var
  Index: Integer;

begin
  with ListViewSCR do
  begin
    for Index := Items.Count - 1 downto 0 do
      if Selected[Index] then
      begin
        OrderListBox.Items.Delete( OrderListBox.Items.IndexOf( Copy( Items[Index], 1, Pos( ':', Items[Index] ) - 1 ) ) );
        Items.Delete( Index );
      end;
    if Items.Count = 0 then
      BitBtnDel.Enabled := False
    else
      BitBtnDel.Enabled := True;
  end;
  BitBtnDel.Enabled := False;
  BitBtnSet.Enabled := False;
  BitBtnTest.Enabled := False;
end;

procedure TRandomSetForm.BitBtnSetClick(Sender: TObject);
var
  s, s2: string;
  i: integer;

begin
  with ListViewSCR do
  begin
    s := Items[ItemIndex];
    i := Length( S );

    repeat
      dec( i );
    until '"' = s[i];

    s := Copy( s, i + 1, Length( s ) - (i+1) );
    if 0 = ParamCount then
      s2 := ' /c'
    else
      s2 := ' ' + ParamStr(1);
    i := WinExec( pChar( s + s2 ), 0 );
    if 32 > i then
      Application.MessageBox( pChar( msgRunError + s + s2 ), capRunError, mb_Ok OR mb_IconError );
  end;
end;

procedure TRandomSetForm.BitBtnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TRandomSetForm.BitBtnOkClick(Sender: TObject);
var
  {Index: Integer;}
  Registry: TAppRegistry;
  S: String;
  i, ii: Integer;

begin
  Registry := TAppRegistry.Create( self );
  try
    with Registry do
    begin
      CompanyName     := sCompanyName;
      ApplicationName := sApplicationName;
      GroupName       := sGroupName;

      Data.Clear;
      for i := 0 to ListViewSCR.Items.Count - 1 do
      begin
        S := ListViewSCR.Items[i];
        ii := Length( S );
        if S[ii] <> '"' then
          raise Exception.Create( 'valid strings has to end with an quotation mark' );

        repeat
          dec( ii );
        until '"' = S[ii];
        Data.Add( Copy( S, ii + 1, Length( S ) - (ii+1) ) );
      end;

      Values.Clear;
      for i := 0 to ListViewSCR.Items.Count - 1 do
      begin
        S := ListViewSCR.Items[i];
        ii := 1;
        repeat
          inc( ii );
        until ':' = S[ii];
        Values.Add( Copy( S, 1, ii-1 ) );
      end;

      SetItems;
      GroupName := sRandomGroup;

      Values.Clear;
      Data.Clear;

      Values.Assign( OrderListBox.Items );
      for i := 0 to OrderListBox.Items.Count - 1 do
      begin
        Str( i, s );
        Data.Add( s );
      end;
      SetItems;
    end;
  finally
    if Registry <> nil then
      Registry.Free;
  end;
  Close;
end;

procedure TRandomSetForm.BitBtnTestClick(Sender: TObject);
var
  s, s2: string;
  i: integer;

begin
  with ListViewSCR do
  begin
    s := Items[ItemIndex];
    i := Length( S );

    repeat
      dec( i );
    until '"' = s[i];

    s := Copy( s, i + 1, Length( s ) - (i+1) );
    s2 := ' /s';
    i := WinExec( pChar( s + s2 ), 0 );
    if 32 > i then
      Application.MessageBox( pChar( msgRunError + s + s2 ), capRunError, mb_Ok OR mb_IconError );
  end;
end;

procedure TRandomSetForm.ListViewClicked(Sender: TObject);
begin
  with ListViewSCR do
  begin
    BitBtnDel.Enabled := SelCount > 0;
    BitBtnTest.Enabled := SelCount = 1;
    BitBtnSet.Enabled := SelCount = 1;
  end;
end;

procedure TRandomSetForm.FormCreate(Sender: TObject);
var
  Registry: TAppRegistry;
  i: Integer;
  tmp: string;

begin
  Registry := TAppRegistry.Create( self );
  try
    with Registry do
    begin
      CompanyName     := sCompanyName;
      ApplicationName := sApplicationName;
      GroupName       := sGroupName;

      GetItems;
      for i := 0 to Data.Count - 1 do
      begin
        tmp := GetScreenSaverName( Data[i], Values[i] );
        if '' <> tmp then
          ListViewSCR.Items.Add( tmp );
      end;
      if ListViewSCR.Items.Count = 0 then
        BitBtnOk.Enabled := False
      else
        BitBtnOk.Enabled := True;

      GroupName := sRandomGroup;
      GetItems;
      for i := 0 to Values.Count - 1 do
      begin
        Str( i, tmp );
        OrderListBox.Items.Insert( i, Values[Data.IndexOf( tmp )] );
      end;
    end;
  finally
    if Registry <> nil then
      Registry.Free;
  end;
end;

procedure TRandomSetForm.OrderListBoxClick(Sender: TObject);
begin
  with OrderListBox do
    if Selected[0] = False then
      BitBtnUp.Enabled := True
    else
      BitBtnUp.Enabled := False;
  with OrderListBox do
    if Selected[Items.Count - 1] = False then
      BitBtnDown.Enabled := True
    else
      BitBtnDown.Enabled := False;
end;

procedure TRandomSetForm.BitBtnRandomClick(Sender: TObject);
var
  PlayList: TStrings;
  i, ii: integer;

begin
  PlayList := TStringList.Create;

  for i := OrderListBox.Items.Count - 1 downto 0 do
  begin
    ii := Random( OrderListBox.Items.Count );
    PlayList.Add( OrderListBox.Items[ii] );
    OrderListBox.Items.Delete( ii );
  end;
  OrderListBox.Items.Assign( PlayList );
  PlayList.Free;
end;

procedure TRandomSetForm.BitBtnSortClick(Sender: TObject);
var
  PlayList: TStrings;

begin
  PlayList := TStringList.Create;
  PlayList.Assign( OrderListBox.Items );
  with PlayList as TStringList do
    Sort;
  OrderListBox.Items.Clear;
  OrderListBox.Items.Assign( PlayList );
  PlayList.Free;

  BitBtnUp.Enabled := False;
  BitBtnDown.Enabled := False;
end;

procedure TRandomSetForm.BitBtnUpClick(Sender: TObject);
var
  s1, s2: string;
  i: integer;

begin
  with OrderListBox do
  begin
    i := ItemIndex;
    s1 := Items[i];
    s2 := Items[i-1];
    Items.Delete( i );
    Items[i-1] := s2;
    Items.Insert( i-1, s1 );
    Dec( i );
    ItemIndex := i;
    
    if i = 0 then
      BitBtnUp.Enabled := False;
    if i < Items.Count - 1 then
      BitBtnDown.Enabled := True;
  end;
end;

procedure TRandomSetForm.BitBtnDownClick(Sender: TObject);
var
  s1, s2: string;
  i: integer;

begin
  with OrderListBox do
  begin
    i := ItemIndex;
    s1 := Items[i];
    s2 := Items[i+1];
    Items.Delete( i );
    Items[i] := s2;
    Items.Insert( i+1, s1 );
    Inc( i );
    ItemIndex := i;

    if i > 0 then
      BitBtnUp.Enabled := True;
    if i = Items.Count - 1 then
      BitBtnDown.Enabled := False;
  end;
end;

procedure TRandomSetForm.TabsChange(Sender: TObject);
begin
  if 0 = OrderListBox.Items.Count then
  begin
    BitBtnSort.Enabled := False;
    BitBtnRandom.Enabled := False;
  end else
  begin
    BitBtnSort.Enabled := True;
    BitBtnRandom.Enabled := True;
  end;
end;

end.
